/* ============================================================================
 * Copyright (c) 1998 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Every modification must be notified to the Java Apache Project
 *    and redistribution of the modified code without prior notification
 *    is NOT permitted in any form.
 *
 * 4. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache Project
 *    (http://java.apache.org/)."
 *
 * 5. The names "Apache JMeter" and "Java Apache Project" must not be used to 
 *    endorse or promote products derived from this software without prior 
 *    written permission.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache Project
 *    (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software was developed by Beta Version Productions and 
 * donated to the Java Apache Project that continues its development.
 * For more information on the Java Apache Project and the JMeter project, 
 * please see <http://java.apache.org/>.
 */

package org.apache.jmeter;

import java.io.*;
import java.net.*;
import org.apache.jmeter.timers.*;
import org.apache.jmeter.visualisers.*;
import javax.swing.*;

/**
 * This class implements the runnable inteface to provide
 * the action that sampling threads should perform to 
 * sample URL resources.
 *
 * It also extends JLabel to visually inform about its status.
 *
 * @author  Stefano Mazzocchi <mazzocch@mbox.systemy.it>
 * @version $Revision: 1.2 $ $Date: 1998/12/14 00:52:04 $
 */
class URLSampler extends JLabel implements Runnable {

    private URL url;
    private String query;
    private boolean alive;
    private org.apache.jmeter.timers.Timer timer;
    private Visualiser visualiser;
    
    public URLSampler(ImageIcon icon) {
        super(icon);
    }
    
    /** 
     * Set the visualiser that currently collects samples
     */
    public void set(Visualiser visualiser) {
        this.visualiser = visualiser;
    }
    
    /** 
     * Initialize the sampler.
     */
    public void set(Visualiser visualiser, org.apache.jmeter.timers.Timer timer, 
        URL url, String query) 
    {
        this.visualiser = visualiser;
        this.timer = timer;
        this.timer.set();
        this.url = url;
        this.query = query;
        this.alive = true;
    } 
    
    /** 
     * Used to try URL access before starting with sampling loop.
     */
    public static URL tryURL(String urlName, String query) throws Exception {
        URL url = new URL(urlName);
        URLConnection connection = url.openConnection();
        connection.setUseCaches(false);
    
        if (query != null) {
            connection.setDoOutput(true);
            PrintWriter output = new PrintWriter(connection.getOutputStream());
            output.println(query);
            output.close();
        }

        connection.getInputStream().close();
        return url;
    }

    /** 
     * Sampling loop and thread execution.
     */
    public void run() {
        while (alive) {
            synchronized (this) {
                try {
                    this.setEnabled(false);
                    this.repaint();
                    wait(timer.delay());
                    this.setEnabled(true);
                    this.repaint();
                } catch (InterruptedException ignored) {}
            }
            
            try {
                long sample = this.sample();
                if (alive) visualiser.add(sample);
            } catch (IOException e) {
                error(e, this);
                this.alive = false;
            }
        }
        
        this.setEnabled(false);
        this.repaint();
    }
    
    /** 
     * Causes this thread to die after finishing its current sample.
     */
    public void die() {
        this.alive = false;
    }
                            
    /** 
     * URL Sampling routine.
     */
    private long sample() throws IOException {
        URLConnection connection = url.openConnection();
        connection.setUseCaches(false);
        if (query != null) connection.setDoOutput(true);
        
        // Get current time
        long time =  System.currentTimeMillis();

        if (query != null) {
            PrintWriter output = new PrintWriter(connection.getOutputStream());
            output.println(query);
            output.close();
        }
        
        BufferedInputStream input = new BufferedInputStream(connection.getInputStream());
        for (int c = 0; c != -1; c = input.read());
        input.close();

        // Get elapsed time
        return System.currentTimeMillis() - time;
    }
    
    /**
     * Method used to signal an Exception to the user.
     */
    public static void error(Exception e, JComponent thrower) {
        JOptionPane.showMessageDialog(thrower, e, "Error", JOptionPane.ERROR_MESSAGE);
    }
}